@@ -1,12 +1,12 @@
-"""Expose the core OSMnx API."""
+"""Expose most common parts of public API directly in `osmnx.` namespace."""
 
 from .bearing import add_edge_bearings
-from .boundaries import gdf_from_place
-from .boundaries import gdf_from_places
 from .distance import get_nearest_edge
 from .distance import get_nearest_edges
 from .distance import get_nearest_node
 from .distance import get_nearest_nodes
+from .distance import k_shortest_paths
+from .distance import shortest_path
 from .elevation import add_edge_grades
 from .elevation import add_node_elevations
 from .folium import plot_graph_folium
@@ -15,6 +15,14 @@ from .footprints import footprints_from_address
 from .footprints import footprints_from_place
 from .footprints import footprints_from_point
 from .footprints import footprints_from_polygon
+from .geocoder import geocode
+from .geocoder import geocode_to_gdf
+from .geometries import geometries_from_address
+from .geometries import geometries_from_bbox
+from .geometries import geometries_from_place
+from .geometries import geometries_from_point
+from .geometries import geometries_from_polygon
+from .geometries import geometries_from_xml
 from .graph import graph_from_address
 from .graph import graph_from_bbox
 from .graph import graph_from_place
@@ -31,11 +39,11 @@ from .plot import plot_footprints
 from .plot import plot_graph
 from .plot import plot_graph_route
 from .plot import plot_graph_routes
-from .plot import plot_shape
 from .pois import pois_from_address
 from .pois import pois_from_place
 from .pois import pois_from_point
 from .pois import pois_from_polygon
+from .projection import project_gdf
 from .projection import project_graph
 from .simplification import consolidate_intersections
 from .simplification import simplify_graph
@@ -47,7 +55,6 @@ from .utils import citation
 from .utils import config
 from .utils import log
 from .utils import ts
-from .utils_geo import geocode
 from .utils_graph import get_undirected
 from .utils_graph import graph_from_gdfs
 from .utils_graph import graph_to_gdfs
