@@ -9,7 +9,9 @@ def get_bearing(origin_point, destination_point):
     """
     Calculate the bearing between two lat-lng points.
 
-    Each tuple should represent (lat, lng) as decimal degrees.
+    Each argument tuple should represent (lat, lng) as decimal degrees.
+    Bearing represents angle in degrees (clockwise) between north and the
+    direction from the origin point to the destination point.
 
     Parameters
     ----------
@@ -21,8 +23,8 @@ def get_bearing(origin_point, destination_point):
     Returns
     -------
     bearing : float
-        the compass bearing in decimal degrees from the origin point
-        to the destination point
+        the compass bearing in decimal degrees from the origin point to the
+        destination point
     """
     if not (isinstance(origin_point, tuple) and isinstance(destination_point, tuple)):
         raise TypeError("origin_point and destination_point must be (lat, lng) tuples")
@@ -37,25 +39,28 @@ def get_bearing(origin_point, destination_point):
     y = math.cos(lat1) * math.sin(lat2) - (math.sin(lat1) * math.cos(lat2) * math.cos(diff_lng))
     initial_bearing = math.atan2(x, y)
 
-    # normalize initial bearing to 0 degrees to 360 degrees to get compass bearing
+    # normalize initial bearing to 0-360 degrees to get compass bearing
     initial_bearing = math.degrees(initial_bearing)
     bearing = (initial_bearing + 360) % 360
 
     return bearing
 
 
-def add_edge_bearings(G):
+def add_edge_bearings(G, precision=1):
     """
-    Add bearing attributes to all graph edges.
+    Add `bearing` attributes to all graph edges.
 
-    Calculate the compass bearing from origin node to destination
-    node for each edge in the directed graph then add each bearing
-    as a new edge attribute.
+    Calculate the compass bearing from origin node to destination node for
+    each edge in the directed graph then add each bearing as a new edge
+    attribute. Bearing represents angle in degrees (clockwise) between north
+    and the direction from the origin node to the destination node.
 
     Parameters
     ----------
     G : networkx.MultiDiGraph
         input graph
+    precision : int
+        decimal precision to round bearing
 
     Returns
     -------
@@ -73,8 +78,6 @@ def add_edge_bearings(G):
             origin_point = (G.nodes[u]["y"], G.nodes[u]["x"])
             destination_point = (G.nodes[v]["y"], G.nodes[v]["x"])
             bearing = get_bearing(origin_point, destination_point)
-
-            # round to thousandth of a degree
-            data["bearing"] = round(bearing, 3)
+            data["bearing"] = round(bearing, precision)
 
     return G
