@@ -1,4 +1,6 @@
-"""Functions to calculate distances and find nearest node/edge(s) to point(s)."""
+"""Calculate distances and shortest paths and find nearest node/edge(s) to point(s)."""
+
+import itertools
 
 import networkx as nx
 import numpy as np
@@ -22,28 +24,29 @@ except ImportError:
 
 def great_circle_vec(lat1, lng1, lat2, lng2, earth_radius=6371009):
     """
-    Calculate great-circle distances.
+    Calculate great-circle distances between points.
 
     Vectorized function to calculate the great-circle distance between two
-    points or between vectors of points, using haversine.
+    points' coordinates or between arrays of points' coordinates using the
+    haversine formula. Expects coordinates in decimal degrees.
 
     Parameters
     ----------
-    lat1 : float or array of float
-        first lat coord
-    lng1 : float or array of float
-        first lng coord
-    lat2 : float or array of float
-        second lat coord
-    lng2 : float or array of float
-        second lng coord
-    earth_radius : numeric
+    lat1 : float or np.array of float
+        first point's latitude coordinate
+    lng1 : float or np.array of float
+        first point's longitude coordinate
+    lat2 : float or np.array of float
+        second point's latitude coordinate
+    lng2 : float or np.array of float
+        second point's longitude coordinate
+    earth_radius : int or float
         radius of earth in units in which distance will be returned
         (default is meters)
 
     Returns
     -------
-    dist : float or np.array of floats
+    dist : float or np.array
         distance or array of distances from (lat1, lng1) to (lat2, lng2) in
         units of earth_radius
     """
@@ -67,40 +70,41 @@ def great_circle_vec(lat1, lng1, lat2, lng2, earth_radius=6371009):
 
 def euclidean_dist_vec(y1, x1, y2, x2):
     """
-    Calculate euclidean distances.
+    Calculate Euclidean distances between points.
 
-    Vectorized function to calculate the euclidean distance between two points
-    or between arrays of points.
+    Vectorized function to calculate the Euclidean distance between two
+    points' coordinates or between arrays of points' coordinates. For most
+    accurate results, use projected coordinates rather than decimal degrees.
 
     Parameters
     ----------
     y1 : float or np.array of float
-        first y coord
+        first point's y coordinate
     x1 : float or np.array of float
-        first x coord
+        first point's x coordinate
     y2 : float or np.array of float
-        second y coord
+        second point's y coordinate
     x2 : float or np.array of float
-        second x coord
+        second point's x coordinate
 
     Returns
     -------
     dist : float or np.array of float
-        distance or vector of distances from (x1, y1) to (x2, y2) in graph
-        units
+        distance or array of distances from (x1, y1) to (x2, y2) in
+        coordinates' units
     """
-    # euclid's formula
+    # Pythagorean theorem
     dist = ((x1 - x2) ** 2 + (y1 - y2) ** 2) ** 0.5
     return dist
 
 
 def get_nearest_node(G, point, method="haversine", return_dist=False):
     """
-    Find node nearest to a point.
+    Find the nearest node to a point.
 
-    Return the graph node nearest to some specified (lat, lng) or (y, x) point,
-    and optionally the distance between the node and the point. This function
-    can use either a haversine or euclidean distance calculator.
+    Return the graph node nearest to some (lat, lng) or (y, x) point and
+    optionally the distance between the node and the point. This function can
+    use either the haversine formula or Euclidean distance.
 
     Parameters
     ----------
@@ -116,61 +120,54 @@ def get_nearest_node(G, point, method="haversine", return_dist=False):
     return_dist : bool
         Optionally also return the distance (in meters if haversine, or graph
         node coordinate units if euclidean) between the point and the nearest
-        node.
+        node
 
     Returns
     -------
     int or tuple of (int, float)
-        Nearest node ID or optionally a tuple of (node ID, dist), where dist is
-        the distance (in meters if haversine, or graph node coordinate units
-        if euclidean) between the point and nearest node
+        Nearest node ID or optionally a tuple of (node ID, dist), where dist
+        is the distance (in meters if haversine, or graph node coordinate
+        units if euclidean) between the point and nearest node
     """
-    if not G or (G.number_of_nodes() == 0):
-        raise ValueError("G argument must be not be empty or should contain at least one node")
+    if len(G) < 1:
+        raise ValueError("G must contain at least one node")
 
     # dump graph node coordinates into a pandas dataframe indexed by node id
     # with x and y columns
-    coords = [[node, data["x"], data["y"]] for node, data in G.nodes(data=True)]
+    coords = ((n, d["x"], d["y"]) for n, d in G.nodes(data=True))
     df = pd.DataFrame(coords, columns=["node", "x", "y"]).set_index("node")
 
-    # add columns to the dataframe representing the (constant) coordinates of
-    # the reference point
-    df["reference_y"] = point[0]
-    df["reference_x"] = point[1]
+    # add columns to df for the (constant) coordinates of reference point
+    df["ref_y"] = point[0]
+    df["ref_x"] = point[1]
 
     # calculate the distance between each node and the reference point
     if method == "haversine":
-        # calculate distance vector using haversine (ie, for
-        # spherical lat-lng geometries)
-        distances = great_circle_vec(
-            lat1=df["reference_y"], lng1=df["reference_x"], lat2=df["y"], lng2=df["x"]
-        )
+        # calculate distances using haversine for spherical lat-lng geometries
+        dists = great_circle_vec(lat1=df["ref_y"], lng1=df["ref_x"], lat2=df["y"], lng2=df["x"])
 
     elif method == "euclidean":
-        # calculate distance vector using euclidean distances (ie, for projected
-        # planar geometries)
-        distances = euclidean_dist_vec(
-            y1=df["reference_y"], x1=df["reference_x"], y2=df["y"], x2=df["x"]
-        )
+        # calculate distances using euclid's formula for projected geometries
+        dists = euclidean_dist_vec(y1=df["ref_y"], x1=df["ref_x"], y2=df["y"], x2=df["x"])
 
     else:
         raise ValueError('method argument must be either "haversine" or "euclidean"')
 
     # nearest node's ID is the index label of the minimum distance
-    nearest_node = distances.idxmin()
+    nearest_node = dists.idxmin()
     utils.log(f"Found nearest node ({nearest_node}) to point {point}")
 
     # if caller requested return_dist, return distance between the point and the
     # nearest node as well
     if return_dist:
-        return nearest_node, distances.loc[nearest_node]
+        return nearest_node, dists.loc[nearest_node]
     else:
         return nearest_node
 
 
 def get_nearest_edge(G, point, return_geom=False, return_dist=False):
     """
-    Return the nearest edge to a point, by minimum euclidean distance.
+    Find the nearest edge to a point by minimum Euclidean distance.
 
     Parameters
     ----------
@@ -183,7 +180,7 @@ def get_nearest_edge(G, point, return_geom=False, return_dist=False):
         Optionally return the geometry of the nearest edge
     return_dist : bool
         Optionally return the distance in graph's coordinates' units between
-        the point and the nearest node
+        the point and the nearest edge
 
     Returns
     -------
@@ -197,7 +194,7 @@ def get_nearest_edge(G, point, return_geom=False, return_dist=False):
     gdf_edges = utils_graph.graph_to_gdfs(G, nodes=False, fill_edge_geometry=True)
     edges = gdf_edges[["u", "v", "key", "geometry"]].values
 
-    # convert lat-lng point to x-y for shapely distance operation
+    # convert lat,lng (y,x) point to x,y for shapely distance operation
     xy_point = Point(reversed(point))
 
     # calculate euclidean distance from each edge's geometry to this point
@@ -220,9 +217,9 @@ def get_nearest_edge(G, point, return_geom=False, return_dist=False):
 
 def get_nearest_nodes(G, X, Y, method=None):
     """
-    Return the graph nodes nearest to a list of points.
+    Find the nearest node to each point in a list of points.
 
-    Pass in points as separate vectors of X and Y coordinates. The 'kdtree'
+    Pass in points as separate lists of X and Y coordinates. The 'kdtree'
     method is by far the fastest with large data sets, but only finds
     approximate nearest nodes if working in unprojected coordinates like
     lat-lng (it precisely finds the nearest node if working in projected
@@ -234,14 +231,14 @@ def get_nearest_nodes(G, X, Y, method=None):
     G : networkx.MultiDiGraph
         input graph
     X : list-like
-        The vector of longitudes or x's for which we will find the nearest
+        the longitudes or x coordinates for which we will find the nearest
         node in the graph
     Y : list-like
-        The vector of latitudes or y's for which we will find the nearest
-        node in the graph
+        the latitudes or y coordinates for which we will find the nearest node
+        in the graph
     method : string {None, 'kdtree', 'balltree'}
-        Which method to use for finding nearest node to each point.
-        If None, we manually find each node one at a time using
+        Which method to use for finding the nearest node to each point. If
+        None, we manually find each node one at a time using
         utils.get_nearest_node and haversine. If 'kdtree' we use
         scipy.spatial.cKDTree for very fast euclidean search. If
         'balltree', we use sklearn.neighbors.BallTree for fast
@@ -250,7 +247,8 @@ def get_nearest_nodes(G, X, Y, method=None):
     Returns
     -------
     nn : np.array
-        list of nearest node IDs
+        array of node IDs representing the node nearest to each point in the
+        passed-in list of points
     """
     if method is None:
 
@@ -259,7 +257,7 @@ def get_nearest_nodes(G, X, Y, method=None):
 
     elif method == "kdtree":
 
-        # check if we were able to import scipy.spatial.cKDTree successfully
+        # check if we were able to import scipy.spatial.cKDTree
         if not cKDTree:
             raise ImportError("The scipy package must be installed to use this optional feature.")
 
@@ -276,13 +274,14 @@ def get_nearest_nodes(G, X, Y, method=None):
 
     elif method == "balltree":
 
-        # check if we were able to import sklearn.neighbors.BallTree successfully
+        # check if we were able to import sklearn.neighbors.BallTree
         if not BallTree:
             raise ImportError(
                 "The scikit-learn package must be installed to use this optional feature."
             )
 
-        # haversine requires data in form of [lat, lng] and inputs/outputs in units of radians
+        # haversine requires data in form of [lat, lng] and inputs/outputs in
+        # units of radians
         nodes = pd.DataFrame(
             {"x": nx.get_node_attributes(G, "x"), "y": nx.get_node_attributes(G, "y")}
         )
@@ -306,9 +305,9 @@ def get_nearest_nodes(G, X, Y, method=None):
 
 def get_nearest_edges(G, X, Y, method=None, dist=0.0001):
     """
-    Return the graph edges nearest to a list of points.
+    Find the nearest edge to each point in a list of points.
 
-    Pass in points as separate vectors of X and Y coordinates. The 'kdtree'
+    Pass in points as separate lists of X and Y coordinates. The 'kdtree'
     method is by far the fastest with large data sets, but only finds
     approximate nearest edges if working in unprojected coordinates like
     lat-lng (it precisely finds the nearest edge if working in projected
@@ -321,40 +320,39 @@ def get_nearest_edges(G, X, Y, method=None, dist=0.0001):
     the X vector corresponds to longitude and the Y vector corresponds
     to latitude. The method creates equally distanced points along the edges
     of the network. Then, these points are used in a kdTree or BallTree search
-    to identify which is nearest.Note that this method will not give the exact
+    to identify which is nearest. Note that this method will not give exact
     perpendicular point along the edge, but the smaller the *dist* parameter,
-    the closer the solution will be.
+    the closer (but slower) the solution will be.
 
     Parameters
     ----------
     G : networkx.MultiDiGraph
         input graph
     X : list-like
-        The vector of longitudes or x's for which we will find the nearest
-        edge in the graph. For projected graphs, use the projected coordinates,
+        the longitudes or x coordinates for which we will find the nearest
+        edge in the graph. For projected graphs use the projected coordinates,
         usually in meters.
     Y : list-like
-        The vector of latitudes or y's for which we will find the nearest
-        edge in the graph. For projected graphs, use the projected coordinates,
+        the latitudes or y coordinates for which we will find the nearest edge
+        in the graph. For projected graphs use the projected coordinates,
         usually in meters.
     method : string {None, 'kdtree', 'balltree'}
-        Which method to use for finding nearest edge to each point.
-        If None, we manually find each edge one at a time using
-        get_nearest_edge. If 'kdtree' we use
-        scipy.spatial.cKDTree for very fast euclidean search. Recommended for
-        projected graphs. If 'balltree', we use sklearn.neighbors.BallTree for
-        fast haversine search. Recommended for unprojected graphs.
-
+        Which method to use for finding nearest edge to each point. If None,
+        we manually find each edge one at a time using get_nearest_edge. If
+        'kdtree' we use scipy.spatial.cKDTree for very fast euclidean search.
+        Recommended for projected graphs. If 'balltree', we use
+        sklearn.neighbors.BallTree for fast haversine search. Recommended for
+        unprojected graphs.
     dist : float
-        spacing length along edges. Units are the same as the geom; Degrees for
-        unprojected geometries and meters for projected geometries. The smaller
-        the value, the more points are created.
+        spacing length along edges. Units are the same as the graph's
+        geometries. The smaller the value, the more points are created.
 
     Returns
     -------
     ne : np.array
-        array of nearest edges represented by their startpoint and endpoint ids,
-        u and v, the OSM ids of the nodes, and the edge key.
+        array of edge IDs representing the edge nearest to each point in the
+        passed-in list of points. Edge IDs are represented by u, v, key where
+        u and v the node IDs of the nodes the edge links.
     """
     if method is None:
         # calculate nearest edge one at a time for each (y, x) point
@@ -453,3 +451,64 @@ def get_nearest_edges(G, X, Y, method=None, dist=0.0001):
     utils.log(f"Found nearest edges to {len(X)} points")
 
     return np.array(ne)
+
+
+def shortest_path(G, orig, dest, weight="length"):
+    """
+    Get shortest path from origin node to destination node.
+
+    See also `k_shortest_paths` to get multiple shortest paths.
+
+    This function is a convenience wrapper around networkx.shortest_path. For
+    more functionality or different algorithms, use networkx directly.
+
+    Parameters
+    ----------
+    G : networkx.MultiDiGraph
+        input graph
+    orig : int
+        origin node ID
+    dest : int
+        destination node ID
+    weight : string
+        edge attribute to minimize when solving shortest path. default is edge
+        length in meters.
+
+    Returns
+    -------
+    path : list
+        list of node IDs consituting the shortest path
+    """
+    path = nx.shortest_path(G, orig, dest, weight=weight)
+    return path
+
+
+def k_shortest_paths(G, orig, dest, k, weight="length"):
+    """
+    Get k shortest paths from origin node to destination node.
+
+    See also `shortest_path` to get just the one shortest path.
+
+    Parameters
+    ----------
+    G : networkx.MultiDiGraph
+        input graph
+    orig : int
+        origin node ID
+    dest : int
+        destination node ID
+    k : int
+        number of shortest paths to get
+    weight : string
+        edge attribute to minimize when solving shortest paths. default is
+        edge length in meters.
+
+    Returns
+    -------
+    generator
+        a generator of k shortest paths ordered by total weight. each path is
+        a list of node IDs.
+    """
+    paths_gen = nx.shortest_simple_paths(utils_graph.get_digraph(G, weight), orig, dest, weight)
+    for path in itertools.islice(paths_gen, 0, k):
+        yield path
