@@ -41,13 +41,10 @@ def save_graph_geopackage(G, filepath=None, encoding="utf-8"):
     if not folder == "" and not os.path.exists(folder):
         os.makedirs(folder)
 
-    # convert undirected graph to geodataframes
+    # convert undirected graph to gdfs and stringify non-numeric columns
     gdf_nodes, gdf_edges = utils_graph.graph_to_gdfs(utils_graph.get_undirected(G))
-
-    # make every non-numeric edge attribute (besides geometry) a string
-    for col in [c for c in gdf_edges.columns if not c == "geometry"]:
-        if not pd.api.types.is_numeric_dtype(gdf_edges[col]):
-            gdf_edges[col] = gdf_edges[col].fillna("").astype(str)
+    gdf_nodes = _stringify_nonnumeric_cols(gdf_nodes)
+    gdf_edges = _stringify_nonnumeric_cols(gdf_edges)
 
     # save the nodes and edges as GeoPackage layers
     gdf_nodes.to_file(filepath, layer="nodes", driver="GPKG", encoding=encoding)
@@ -59,6 +56,10 @@ def save_graph_shapefile(G, filepath=None, encoding="utf-8"):
     """
     Save graph nodes and edges to disk as ESRI shapefiles.
 
+    The shapefile format is proprietary and outdated. Whenever possible, you
+    should use the superior GeoPackage file format instead, for instance, via
+    the save_graph_geopackage function.
+
     Parameters
     ----------
     G : networkx.MultiDiGraph
@@ -84,18 +85,10 @@ def save_graph_shapefile(G, filepath=None, encoding="utf-8"):
     filepath_nodes = os.path.join(filepath, "nodes.shp")
     filepath_edges = os.path.join(filepath, "edges.shp")
 
-    # convert undirected graph to geodataframes
+    # convert undirected graph to gdfs and stringify non-numeric columns
     gdf_nodes, gdf_edges = utils_graph.graph_to_gdfs(utils_graph.get_undirected(G))
-
-    # make every non-numeric edge attribute (besides geometry) a string
-    for col in [c for c in gdf_edges.columns if not c == "geometry"]:
-        if not pd.api.types.is_numeric_dtype(gdf_edges[col]):
-            gdf_edges[col] = gdf_edges[col].fillna("").astype(str)
-
-    # make every non-numeric node attribute (besides geometry) a string
-    for col in [c for c in gdf_nodes.columns if not c == "geometry"]:
-        if not pd.api.types.is_numeric_dtype(gdf_nodes[col]):
-            gdf_nodes[col] = gdf_nodes[col].fillna("").astype(str)
+    gdf_nodes = _stringify_nonnumeric_cols(gdf_nodes)
+    gdf_edges = _stringify_nonnumeric_cols(gdf_edges)
 
     # save the nodes and edges as separate ESRI shapefiles
     gdf_nodes.to_file(filepath_nodes, encoding=encoding)
@@ -133,14 +126,12 @@ def save_graphml(G, filepath=None, gephi=False, encoding="utf-8"):
     if not folder == "" and not os.path.exists(folder):
         os.makedirs(folder)
 
-    # create a copy to convert all the node/edge attribute values to string
-    G_save = G.copy()
+    # make a copy to not edit the original graph object the caller passed in
+    G = G.copy()
 
     if gephi:
 
-        gdf_nodes, gdf_edges = utils_graph.graph_to_gdfs(
-            G_save, nodes=True, edges=True, node_geometry=True, fill_edge_geometry=True
-        )
+        gdf_nodes, gdf_edges = utils_graph.graph_to_gdfs(G)
 
         # turn each edge's key into a unique ID for Gephi compatibility
         gdf_edges["key"] = range(len(gdf_edges))
@@ -148,33 +139,33 @@ def save_graphml(G, filepath=None, gephi=False, encoding="utf-8"):
         # gephi doesn't handle node attrs named x and y well, so rename
         gdf_nodes["xcoord"] = gdf_nodes["x"]
         gdf_nodes["ycoord"] = gdf_nodes["y"]
-        G_save = utils_graph.graph_from_gdfs(gdf_nodes, gdf_edges)
+        G = utils_graph.graph_from_gdfs(gdf_nodes, gdf_edges)
 
         # remove graph attributes as Gephi only accepts node and edge attrs
-        G_save.graph = {}
+        G.graph = dict()
 
     else:
         # if not gephi, keep graph attrs and stringify them
-        for dict_key in G_save.graph:
+        for dict_key in G.graph:
             # convert all the graph attribute values to strings
-            G_save.graph[dict_key] = str(G_save.graph[dict_key])
+            G.graph[dict_key] = str(G.graph[dict_key])
 
     # stringify node and edge attributes
-    for _, data in G_save.nodes(data=True):
+    for _, data in G.nodes(data=True):
         for dict_key in data:
-            if gephi and dict_key in ["xcoord", "ycoord"]:
+            if gephi and dict_key in {"xcoord", "ycoord"}:
                 # don't convert x y values to string if saving for gephi
                 continue
             else:
                 # convert all the node attribute values to strings
                 data[dict_key] = str(data[dict_key])
 
-    for _, _, data in G_save.edges(keys=False, data=True):
+    for _, _, data in G.edges(keys=False, data=True):
         for dict_key in data:
             # convert all the edge attribute values to strings
             data[dict_key] = str(data[dict_key])
 
-    nx.write_graphml(G_save, path=filepath, encoding=encoding)
+    nx.write_graphml(G, path=filepath, encoding=encoding)
     utils.log(f'Saved graph as GraphML file at "{filepath}"')
 
 
@@ -182,7 +173,7 @@ def load_graphml(filepath, node_type=int):
     """
     Load an OSMnx-saved GraphML file from disk.
 
-    Then convert the node/edge attributes to appropriate data types.
+    Converts the node/edge attributes to appropriate data types.
 
     Parameters
     ----------
@@ -217,7 +208,7 @@ def load_graphml(filepath, node_type=int):
     if "edge_default" in G.graph:
         del G.graph["edge_default"]
 
-    utils.log(f'Loaded graph with {len(G)} nodes and {len(G.edges())} edges from "{filepath}"')
+    utils.log(f'Loaded graph with {len(G)} nodes and {len(G.edges)} edges from "{filepath}"')
     return G
 
 
@@ -230,23 +221,21 @@ def _convert_node_attr_types(G, node_type):
     G : networkx.MultiDiGraph
         input graph
     node_type : type
-        convert osmid to this type
+        convert node ID (osmid) to this type
 
     Returns
     -------
     G : networkx.MultiDiGraph
     """
-    # convert numeric node tags from string to numeric data types
     for _, data in G.nodes(data=True):
+
+        # convert node ID to user-requested type
         data["osmid"] = node_type(data["osmid"])
-        data["x"] = float(data["x"])
-        data["y"] = float(data["y"])
 
-        if "elevation" in data:
-            data["elevation"] = float(data["elevation"])
-
-        if "elevation_res" in data:
-            data["elevation_res"] = float(data["elevation_res"])
+        # convert numeric node attributes from string to float
+        for attr in {"elevation", "elevation_res", "lat", "lon", "x", "y"}:
+            if attr in data:
+                data[attr] = float(data[attr])
 
     return G
 
@@ -274,22 +263,25 @@ def _convert_edge_attr_types(G, node_type):
 
         try:
             data["oneway"] = ast.literal_eval(data["oneway"])
-        except KeyError:
-            # may lack oneway if settings.all_oneway=True when you created graph
-            pass
-        except ValueError:
-            # may have values it can't eval if settings.all_oneway=True
+            raise ValueError()
+        except (KeyError, ValueError):
+            # may lack oneway if settings.all_oneway=True when you created
+            # graph, or values it can't eval if settings.all_oneway=True
             pass
 
-        # parse grade attrs to float: should always have only 1 value each
-        if "grade" in data:
-            data["grade"] = float(data["grade"])
-        if "grade_abs" in data:
-            data["grade_abs"] = float(data["grade_abs"])
+        # convert to float any possible OSMnx-added edge attributes, which may
+        # have multiple values if graph was simplified after they were added
+        for attr in {"grade", "grade_abs", "bearing", "speed_kph", "travel_time"}:
+            if attr in data:
+                if data[attr].startswith("[") and data[attr].endswith("]"):
+                    # if it's a list, eval it then convert each item to float
+                    data[attr] = [float(a) for a in ast.literal_eval(data[attr])]
+                else:
+                    data[attr] = float(data[attr])
 
         # these attributes might have a single value, or a list if edge's
         # topology was simplified
-        for attr in [
+        for attr in {
             "highway",
             "name",
             "bridge",
@@ -303,7 +295,7 @@ def _convert_edge_attr_types(G, node_type):
             "landuse",
             "width",
             "est_width",
-        ]:
+        }:
             # if this edge has this attribute, and it starts with '[' and ends
             # with ']', then it's a list to be parsed
             if attr in data and data[attr].startswith("[") and data[attr].endswith("]"):
@@ -328,9 +320,38 @@ def _convert_edge_attr_types(G, node_type):
         if "geometry" in data:
             data["geometry"] = wkt.loads(data["geometry"])
 
+        # delete the extraneous id attribute added by graphml saving
+        if "id" in data:
+            del data["id"]
+
     return G
 
 
+def _stringify_nonnumeric_cols(gdf):
+    """
+    Make every non-numeric GeoDataFrame column (besides geometry) a string.
+
+    This allows proper serializing via Fiona of GeoDataFrames with mixed types
+    such as strings and ints in the same column.
+
+    Parameters
+    ----------
+    gdf : geopandas.GeoDataFrame
+        gdf to stringify non-numeric columns of
+
+    Returns
+    -------
+    gdf : geopandas.GeoDataFrame
+        gdf with non-numeric columns stringified
+    """
+    # stringify every non-numeric column other than geometry column
+    for col in (c for c in gdf.columns if not c == "geometry"):
+        if not pd.api.types.is_numeric_dtype(gdf[col]):
+            gdf[col] = gdf[col].fillna("").astype(str)
+
+    return gdf
+
+
 def save_graph_xml(
     data,
     filepath=None,
@@ -345,7 +366,29 @@ def save_graph_xml(
     """
     Save graph to disk as an OSM-formatted XML .osm file.
 
-    Note: for large networks this function can take a long time to run.
+    This function exists only to allow serialization to the .osm file format
+    for applications that require it, and has constraints to conform to that.
+    To save/load full-featured OSMnx graphs to/from disk for later use, use
+    the save_graphml and load_graphml functions instead.
+
+    Note: for large networks this function can take a long time to run. Before
+    using this function, make sure you configured OSMnx as described in the
+    example below when you created the graph.
+
+    Example
+    -------
+    >>> import osmnx as ox
+    >>> utn = ox.settings.useful_tags_node
+    >>> oxna = ox.settings.osm_xml_node_attrs
+    >>> oxnt = ox.settings.osm_xml_node_tags
+    >>> utw = ox.settings.useful_tags_way
+    >>> oxwa = ox.settings.osm_xml_way_attrs
+    >>> oxwt = ox.settings.osm_xml_way_tags
+    >>> utn = list(set(utn + oxna + oxnt))
+    >>> utw = list(set(utw + oxwa + oxwt))
+    >>> ox.config(all_oneway=True, useful_tags_node=utn, useful_tags_way=utw)
+    >>> G = ox.graph_from_place('Piedmont, CA, USA', network_type='drive')
+    >>> ox.save_graph_xml(G, filepath='./data/graph1.osm')
 
     Parameters
     ----------
@@ -416,7 +459,7 @@ def save_graph_xml(
         gdf_edges = gdf_edges.reset_index().rename(columns={"index": "id"})
 
     # add default values for required attributes
-    for table in [gdf_nodes, gdf_edges]:
+    for table in (gdf_nodes, gdf_edges):
         table["uid"] = "1"
         table["user"] = "osmnx"
         table["version"] = "1"
@@ -534,7 +577,7 @@ def _append_edges_xml_tree(root, gdf_edges, edge_attrs, edge_tags, edge_tag_aggs
             else:
                 for tag in edge_tags:
                     if (tag in all_way_edges.columns) and (
-                        tag not in [t for t, agg in edge_tag_aggs]
+                        tag not in (t for t, agg in edge_tag_aggs)
                     ):
                         etree.SubElement(edge, "tag", attrib={"k": tag, "v": first[tag]})
 
