@@ -21,7 +21,7 @@ def _is_endpoint(G, node, strict=True):
     to help streets bend around curves. An end point is a node that either:
     1) is its own neighbor, ie, it self-loops.
     2) or, has no incoming edges or no outgoing edges, ie, all its incident
-       edges point inward or all its incident edges point outward.
+    edges point inward or all its incident edges point outward.
     3) or, it does not have exactly two neighbors and degree of 2 or 4.
     4) or, if strict mode is false, if its edges have different OSM IDs.
 
@@ -88,58 +88,81 @@ def _is_endpoint(G, node, strict=True):
         return False
 
 
-def _build_path(G, node, endpoints, path):
+def _build_path(G, endpoint, endpoint_successor, endpoints):
     """
-    Recursively build a path of nodes until you hit an endpoint node.
+    Build a path of nodes from one endpoint node to next endpoint node.
 
     Parameters
     ----------
     G : networkx.MultiDiGraph
         input graph
-    node : int
-        the current node to start from
+    endpoint : int
+        the endpoint node from which to start the path
+    endpoint_successor : int
+        the successor of endpoint through which the path to the next endpoint
+        will be built
     endpoints : set
         the set of all nodes in the graph that are endpoints
-    path : list
-        the list of nodes in order in the path so far
 
     Returns
     -------
     path : list
+        the first and last items in the resulting path list are endpoint
+        nodes, and all other items are interstitial nodes that can be removed
+        subsequently
     """
-    # for each successor in the passed-in node
-    for successor in G.successors(node):
+    # start building path from endpoint node through its successor
+    path = [endpoint, endpoint_successor]
+
+    # for each successor of the endpoint's successor
+    for successor in G.successors(endpoint_successor):
         if successor not in path:
             # if this successor is already in the path, ignore it, otherwise add
             # it to the path
             path.append(successor)
-            if successor not in endpoints:
-                # if this successor is not an endpoint, recursively call
-                # build_path until you find an endpoint
-                path = _build_path(G, successor, endpoints, path)
-            else:
-                # if this successor is an endpoint, we've completed the path,
-                # so return it
-                return path
+            while successor not in endpoints:
+                # find successors (of current successor) not in path
+                successors = [n for n in G.successors(successor) if n not in path]
 
-    if (path[-1] not in endpoints) and (path[0] in G.successors(path[-1])):
-        # if the end of the path is not actually an endpoint and the path's
-        # first node is a successor of the path's final node, then this is
-        # actually a self loop, so add path's first node to end of path to
-        # close it
-        path.append(path[0])
+                # 99%+ of the time there will be only 1 successor: add to path
+                if len(successors) == 1:
+                    successor = successors[0]
+                    path.append(successor)
 
+                # handle relatively rare cases or OSM digitization quirks
+                elif len(successors) == 0:
+                    if endpoint in G.successors(successor):
+                        # we have come to the end of a self-looping edge, so
+                        # add first node to end of path to close it and return
+                        return path + [endpoint]
+                    else:
+                        # this can happen due to OSM digitization error where
+                        # a one-way street turns into a two-way here, but
+                        # duplicate incoming one-way edges are present
+                        utils.log(
+                            f"Unexpected simplify pattern handled near {successor}", level=lg.WARN
+                        )
+                        return path
+                else:
+                    # if successor has >1 successors, then successor must have
+                    # been an endpoint because you can go in 2 new directions.
+                    # this should never occur in practice
+                    raise Exception(f"Unexpected simplify pattern failed near {successor}")
+
+            # if this successor is an endpoint, we've completed the path
+            return path
+
+    # if endpoint_successor has no successors not already in the path, return
+    # the current path: this is usually due to a digitization quirk on OSM
     return path
 
 
 def _get_paths_to_simplify(G, strict=True):
     """
-    Create a list of all the paths to be simplified between endpoint nodes.
+    Generate all the paths to be simplified between endpoint nodes.
 
     The path is ordered from the first endpoint, through the interstitial nodes,
-    to the second endpoint. If your street network is in a rural area with many
-    interstitial nodes between true edge endpoints, you may want to increase
-    your system's recursion limit to avoid recursion errors.
+    to the second endpoint.
 
     Parameters
     ----------
@@ -149,47 +172,28 @@ def _get_paths_to_simplify(G, strict=True):
         if False, allow nodes to be end points even if they fail all other rules
         but have edges with different OSM IDs
 
-    Returns
-    -------
-    paths_to_simplify : list
+    Yields
+    ------
+    path_to_simplify : list
     """
     # first identify all the nodes that are endpoints
-    endpoints = set([node for node in G.nodes() if _is_endpoint(G, node, strict=strict)])
+    endpoints = set([n for n in G.nodes() if _is_endpoint(G, n, strict=strict)])
     utils.log(f"Identified {len(endpoints)} edge endpoints")
 
-    paths_to_simplify = []
-
     # for each endpoint node, look at each of its successor nodes
-    for node in endpoints:
-        for successor in G.successors(node):
+    for endpoint in endpoints:
+        for successor in G.successors(endpoint):
             if successor not in endpoints:
-                # if the successor is not an endpoint, build a path from the
-                # endpoint node to the next endpoint node
-                try:
-                    path = _build_path(G, successor, endpoints, path=[node, successor])
-                    paths_to_simplify.append(path)
-                except RecursionError:
-                    utils.log(
-                        "Exceeded max depth, moving on to next endpoint successor", level=lg.WARNING
-                    )
-                    # recursion errors occur if some connected component is a
-                    # self-contained ring in which all nodes are not end points.
-                    # could also occur in extremely long street segments (eg, in
-                    # rural areas) with too many nodes between true endpoints.
-                    # handle it by just ignoring that component and letting its
-                    # topology remain intact (this should be a rare occurrence)
-
-    utils.log("Constructed all paths to simplify")
-    return paths_to_simplify
+                # if endpoint node's successor is not an endpoint, build a path
+                # from the endpoint node, through the successor, and on to the
+                # next endpoint node
+                yield _build_path(G, endpoint, successor, endpoints)
 
 
 def _is_simplified(G):
     """
     Determine if a graph has already had its topology simplified.
 
-    If any of its edges have a geometry attribute, we know that it has
-    previously been simplified.
-
     Parameters
     ----------
     G : networkx.MultiDiGraph
@@ -202,14 +206,16 @@ def _is_simplified(G):
     return "simplified" in G.graph and G.graph["simplified"]
 
 
-def simplify_graph(G, strict=True):
+def simplify_graph(G, strict=True, remove_rings=True):
     """
     Simplify a graph's topology by removing interstitial nodes.
 
     Simplify graph topology by removing all nodes that are not intersections
     or dead-ends. Create an edge directly between the end points that
     encapsulate them, but retain the geometry of the original edges, saved as
-    attribute in new edge.
+    an attribute in new edge. Some of the resulting consolidated edges may
+    comprise multiple OSM ways, and if so, their multiple attribute values are
+    stored as a list.
 
     Parameters
     ----------
@@ -217,7 +223,11 @@ def simplify_graph(G, strict=True):
         input graph
     strict : bool
         if False, allow nodes to be end points even if they fail all other
-        rules but have edges with different OSM IDs
+        rules but have incident edges with different OSM IDs. Lets you keep
+        nodes at elbow two-way intersections, but sometimes individual blocks
+        have multiple OSM IDs within them too.
+    remove_rings : bool
+        if True, remove isolated self-contained rings that have no endpoints
 
     Returns
     -------
@@ -228,32 +238,30 @@ def simplify_graph(G, strict=True):
         raise Exception("This graph has already been simplified, cannot simplify it again.")
 
     utils.log("Begin topologically simplifying the graph...")
+
+    # make a copy to not edit the original graph object the caller passed in
     G = G.copy()
-    initial_node_count = len(list(G.nodes()))
-    initial_edge_count = len(list(G.edges()))
+    initial_node_count = len(G)
+    initial_edge_count = len(G.edges)
     all_nodes_to_remove = []
     all_edges_to_add = []
 
-    # construct a list of all the paths that need to be simplified
-    paths = _get_paths_to_simplify(G, strict=strict)
-
-    for path in paths:
+    # generate each path that needs to be simplified
+    for path in _get_paths_to_simplify(G, strict=strict):
 
         # add the interstitial edges we're removing to a list so we can retain
         # their spatial geometry
-        edge_attributes = {}
+        edge_attributes = dict()
         for u, v in zip(path[:-1], path[1:]):
 
-            # there shouldn't be multiple edges between interstitial nodes
+            # there should rarely be multiple edges between interstitial nodes
+            # usually happens if OSM has duplicate ways digitized for just one
+            # street... we will keep only one of the edges (see below)
             if not G.number_of_edges(u, v) == 1:
-                utils.log(
-                    f'Multiple edges between "{u}" and "{v}" found when simplifying',
-                    level=lg.WARNING,
-                )
+                utils.log(f"Found multiple edges between {u} and {v} when simplifying")
 
-            # the only element in this list as long as above check is True
-            # (MultiGraphs use keys (the 0 here), indexed with ints from 0 and
-            # up)
+            # get edge between these nodes: if multiple edges exist between
+            # them (see above), we retain only one in the simplified graph
             edge = G.edges[u, v, 0]
             for key in edge:
                 if key in edge_attributes:
@@ -295,58 +303,77 @@ def simplify_graph(G, strict=True):
     # finally remove all the interstitial nodes between the new edges
     G.remove_nodes_from(set(all_nodes_to_remove))
 
+    if remove_rings:
+        # remove any connected components that form a self-contained ring
+        # without any endpoints
+        wccs = nx.weakly_connected_components(G)
+        nodes_in_rings = set()
+        for wcc in wccs:
+            if all([not _is_endpoint(G, n) for n in wcc]):
+                nodes_in_rings.update(wcc)
+        G.remove_nodes_from(nodes_in_rings)
+
+    # mark graph as having been simplified
     G.graph["simplified"] = True
 
     msg = (
         f"Simplified graph: {initial_node_count} to {len(G)} nodes, "
-        f"{initial_edge_count} to {len(G.edges())} edges"
+        f"{initial_edge_count} to {len(G.edges)} edges"
     )
     utils.log(msg)
     return G
 
 
 def consolidate_intersections(
-    G, tolerance=10, rebuild_graph=True, dead_ends=False, update_edge_lengths=True
+    G, tolerance=10, rebuild_graph=True, dead_ends=False, reconnect_edges=True
 ):
     """
     Consolidate intersections comprising clusters of nearby nodes.
 
-    Merging nodes and return either their centroids or a rebuilt graph with
-    consolidated intersections and reconnected edge geometries.
+    Merges nearby nodes and returns either their centroids or a rebuilt graph
+    with consolidated intersections and reconnected edge geometries. The
+    tolerance argument should be adjusted to approximately match street design
+    standards in the specific street network, and you should always use a
+    projected graph to work in meaningful and consistent units like meters.
 
-    The tolerance argument should be adjusted to approximately match street
-    design standards in the specific street network, and you should always use
-    a projected graph to work in meaningful and consistent units like meters.
+    When rebuild_graph=False, it uses a purely geometrical (and relatively
+    fast) algorithm to identify "geometrically close" nodes, merge them, and
+    return just the merged intersections' centroids. When rebuild_graph=True,
+    it uses a topological (and slower but more accurate) algorithm to identify
+    "topologically close" nodes, merge them, then rebuild/return the graph.
+    Returned graph's node IDs represent clusters rather than osmids. Refer to
+    nodes' osmid attributes for original osmids. If multiple nodes were merged
+    together, the osmid attribute is a list of merged nodes' osmids.
 
     Divided roads are often represented by separate centerline edges. The
     intersection of two divided roads thus creates 4 nodes, representing where
-    each edge intersects a perpendicular edge. These 4 nodes represent a single
-    intersection in the real world. This function consolidates them up by
-    buffering them to an arbitrary distance, merging overlapping buffers, and
-    taking their centroid. For best results, the tolerance argument should be
-    adjusted to approximately match street design standards in the specific
-    street network.
+    each edge intersects a perpendicular edge. These 4 nodes represent a
+    single intersection in the real world. A similar situation occurs with
+    roundabouts and traffic circles. This function consolidates nearby nodes
+    by buffering them to an arbitrary distance, merging overlapping buffers,
+    and taking their centroid.
 
     Parameters
     ----------
     G : networkx.MultiDiGraph
         a projected graph
     tolerance : float
-        nodes within this distance (in graph's geometry's units) will be
-        dissolved into a single intersection
+        nodes are buffered to this distance (in graph's geometry's units) and
+        subsequent overlaps are dissolved into a single node
     rebuild_graph : bool
-        if True, use consolidate_intersections_rebuild_graph to consolidate
-        the intersections and rebuild the graph, then return as
-        networkx.MultiDiGraph. if False, just return the consolidated
-        intersection points as a geopandas.GeoSeries (faster than rebuilding
-        graph)
+        if True, consolidate the nodes topologically, rebuild the graph, and
+        return as networkx.MultiDiGraph. if False, consolidate the nodes
+        geometrically and return the consolidated node points as
+        geopandas.GeoSeries
     dead_ends : bool
         if False, discard dead-end nodes to return only street-intersection
         points
-    update_edge_lengths : bool
-        just passed to consolidate_intersections_rebuild_graph.
-        if True, update the length attribute of edges reconnected to a new
-        merged node; if False, just retain the original edge length.
+    reconnect_edges : bool
+        ignored if rebuild_graph is not True. if True, reconnect edges and
+        their geometries in rebuilt graph to the consolidated nodes and update
+        edge length attributes; if False, returned graph has no edges (which
+        is faster if you just need topologically consolidated intersection
+        counts).
 
     Returns
     -------
@@ -364,29 +391,39 @@ def consolidate_intersections(
             streets_per_node = utils_graph.count_streets_per_node(G)
 
         dead_end_nodes = [node for node, count in streets_per_node.items() if count <= 1]
+
+        # make a copy to not edit the original graph object the caller passed in
         G = G.copy()
         G.remove_nodes_from(dead_end_nodes)
 
     if rebuild_graph:
-        return _consolidate_intersections_rebuild_graph(
-            G=G, tolerance=tolerance, update_edge_lengths=update_edge_lengths
-        )
+        if len(G) == 0 or len(G.edges) == 0:
+            # cannot rebuild a graph with no nodes or no edges, just return it
+            return G
+        else:
+            return _consolidate_intersections_rebuild_graph(
+                G=G, tolerance=tolerance, reconnect_edges=reconnect_edges
+            )
 
     else:
-        # create a GeoDataFrame of nodes, buffer to passed-in distance, merge overlaps
-        gdf_nodes = utils_graph.graph_to_gdfs(G, edges=False)
-        buffered_nodes = gdf_nodes.buffer(tolerance).unary_union
-        if isinstance(buffered_nodes, Polygon):
-            # if only a single node results, make iterable to convert to GeoSeries
-            buffered_nodes = [buffered_nodes]
+        crs = G.graph["crs"]
+        if len(G) == 0:
+            # if graph has no nodes, just return empty GeoSeries
+            return gpd.GeoSeries(crs=crs)
+        else:
+            # create nodes gdf, buffer to passed-in distance, merge overlaps
+            gdf_nodes = utils_graph.graph_to_gdfs(G, edges=False)
+            merged_nodes = gdf_nodes.buffer(tolerance).unary_union
+            if isinstance(merged_nodes, Polygon):
+                # if only a single node results, make iterable to convert to GeoSeries
+                merged_nodes = [merged_nodes]
 
-        # get the centroids of the merged intersection polygons
-        unified_intersections = gpd.GeoSeries(list(buffered_nodes))
-        intersection_centroids = unified_intersections.centroid
-        return intersection_centroids
+            # get the centroids of the merged intersection polygons
+            intersection_centroids = gpd.GeoSeries(list(merged_nodes), crs=crs).centroid
+            return intersection_centroids
 
 
-def _consolidate_intersections_rebuild_graph(G, tolerance=10, update_edge_lengths=True):
+def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=True):
     """
     Consolidate intersections comprising clusters of nearby nodes.
 
@@ -397,8 +434,8 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, update_edge_length
     design standards in the specific street network, and you should always use
     a projected graph to work in meaningful and consistent units like meters.
 
-    Returned graph's node IDs represent clusters rather than OSMIDs. Refer to
-    nodes' osmid attributes for original OSMIDs. If multiple nodes were merged
+    Returned graph's node IDs represent clusters rather than osmids. Refer to
+    nodes' osmid attributes for original osmids. If multiple nodes were merged
     together, the osmid attribute is a list of merged nodes' osmids.
 
     Parameters
@@ -406,12 +443,14 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, update_edge_length
     G : networkx.MultiDiGraph
         a projected graph
     tolerance : float
-        nodes within this distance (in graph's geometry's units) will be
-        dissolved into a single node, with edges reconnected to this new
-        node
-    update_edge_lengths : bool
-        if True, update the length attribute of edges reconnected to a new
-        merged node; if False, just retain the original edge length
+        nodes are buffered to this distance (in graph's geometry's units) and
+        subsequent overlaps are dissolved into a single node
+    reconnect_edges : bool
+        ignored if rebuild_graph is not True. if True, reconnect edges and
+        their geometries in rebuilt graph to the consolidated nodes and update
+        edge length attributes; if False, returned graph has no edges (which
+        is faster if you just need topologically consolidated intersection
+        counts).
 
     Returns
     -------
@@ -421,8 +460,7 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, update_edge_length
     """
     # STEP 1
     # buffer nodes to passed-in distance, merge overlaps
-    gdf_nodes, gdf_edges = utils_graph.graph_to_gdfs(G)
-    gdf_edges = gdf_edges.set_index(["u", "v", "key"])
+    gdf_nodes = utils_graph.graph_to_gdfs(G, edges=False)
     buffered_nodes = gdf_nodes.buffer(tolerance).unary_union
     if isinstance(buffered_nodes, Polygon):
         # if only a single node results, make iterable to convert to GeoSeries
@@ -490,9 +528,14 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, update_edge_length
                 y=nodes_subset["y"].iloc[0],
             )
 
+    if len(G.edges) == 0 or not reconnect_edges:
+        # if reconnect_edges is False or there are no edges in original graph
+        # (after dead-end removed), then skip edges and return new graph as-is
+        return H
+
     # STEP 6
-    # create a new edge for each edge in original graph
-    # but from cluster to cluster
+    # create new edge from cluster to cluster for each edge in original graph
+    gdf_edges = utils_graph.graph_to_gdfs(G, nodes=False).set_index(["u", "v", "key"])
     for u, v, k, data in G.edges(keys=True, data=True):
         u2 = gdf.loc[u, "cluster"]
         v2 = gdf.loc[v, "cluster"]
@@ -507,8 +550,8 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, update_edge_length
             H.add_edge(u2, v2, **data)
 
     # STEP 7
-    # for every group of merged nodes with more than 1 node in it,
-    # extend the edge geometries to reach the new node point
+    # for every group of merged nodes with more than 1 node in it, extend the
+    # edge geometries to reach the new node point
     new_edges = utils_graph.graph_to_gdfs(H, nodes=False)
     for cluster_label, nodes_subset in groups:
 
@@ -531,9 +574,7 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, update_edge_length
                 new_geom = LineString(new_coords)
                 H.edges[u, v, k]["geometry"] = new_geom
 
-                # update the edge length attribute if parameterized to do so
-                # otherwise just keep using the original edge length
-                if update_edge_lengths:
-                    H.edges[u, v, k]["length"] = new_geom.length
+                # update the edge length attribute, given the new geometry
+                H.edges[u, v, k]["length"] = new_geom.length
 
     return H
