@@ -88,11 +88,15 @@ def config(
     log_name=settings.log_name,
     log_filename=settings.log_filename,
     useful_tags_node=settings.useful_tags_node,
-    useful_tags_path=settings.useful_tags_path,
+    useful_tags_way=settings.useful_tags_way,
     osm_xml_node_attrs=settings.osm_xml_node_attrs,
     osm_xml_node_tags=settings.osm_xml_node_tags,
     osm_xml_way_attrs=settings.osm_xml_way_attrs,
     osm_xml_way_tags=settings.osm_xml_way_tags,
+    overpass_settings=settings.overpass_settings,
+    timeout=settings.timeout,
+    memory=settings.memory,
+    max_query_area_size=settings.max_query_area_size,
     default_access=settings.default_access,
     default_crs=settings.default_crs,
     default_user_agent=settings.default_user_agent,
@@ -102,52 +106,68 @@ def config(
     nominatim_key=settings.nominatim_key,
     overpass_endpoint=settings.overpass_endpoint,
     all_oneway=settings.all_oneway,
+    elevation_provider=settings.elevation_provider,
 ):
     """
     Configure OSMnx by setting the default global settings' values.
 
-    Note that any parameters not passed-in by the caller are set to their
-    default values.
+    Any parameters not passed by the caller are set to their original default
+    values.
 
     Parameters
     ----------
     data_folder : string
-        where to save and load data files
+        path to folder in which to save/load data files by default
     logs_folder : string
-        where to write the log files
+        path to folder in which to save log files
     imgs_folder : string
-        where to save figures
+        path to folder in which to save plotted figures by default
     cache_folder : string
-        where to save the http response cache
+        path to folder in which to save/load HTTP response cache
     use_cache : bool
-        if True, use a local cache to save/retrieve http responses instead of
-        calling API repetitively for the same request URL
+        if True, cache HTTP responses locally instead of calling API
+        repetitively for the same request
     log_file : bool
-        if true, save log output to a log file in logs_folder
+        if True, save log output to a file in logs_folder
     log_console : bool
-        if true, print log output to the console
+        if True, print log output to the console (terminal window)
     log_level : int
         one of the logger.level constants
     log_name : string
         name of the logger
     log_filename : string
-        filename of the log
+        name of the log file
     useful_tags_node : list
-        a list of useful OSM tags to attempt to save from node elements
-    useful_tags_path : list
-        a list of useful OSM tags to attempt to save from path elements
+        OSM "node" tags to add as graph node attributes, when present
+    useful_tags_way : list
+        OSM "way" tags to add as graph edge attributes, when present
     osm_xml_node_attrs : list
-        list of node attributes for .osm xml files
+        node attributes for saving .osm XML files with save_graph_xml function
     osm_xml_node_tags : list
-        list of node tags for .osm xml files
+        node tags for saving .osm XML files with save_graph_xml function
     osm_xml_way_attrs : list
-        list of edge attributes for .osm xml files
+        edge attributes for saving .osm XML files with save_graph_xml function
     osm_xml_way_tags : list
-        list of edge tags for .osm xml files
+        edge tags for for saving .osm XML files with save_graph_xml function
+    overpass_settings : string
+        Settings string for overpass queries. For example, to query historical
+        OSM data as of a certain date:
+        `'[out:json][timeout:90][date:"2019-10-28T19:20:00Z"]'`.
+        Use with caution.
+    timeout : int
+        the timeout interval for the HTTP request and for API to use while
+        running the query
+    memory : int
+        Overpass server memory allocation size for the query, in bytes. If
+        None, server will use its default allocation size. Use with caution.
+    max_query_area_size : int
+        maximum area for any part of the geometry in meters: any polygon
+        bigger than this will get divided up for multiple queries to API
+        (default 50km x 50km)
     default_access : string
         default filter for OSM "access" key
     default_crs : string
-        default CRS to set when creating graphs
+        default coordinate reference system to set when creating graphs
     default_user_agent : string
         HTTP header user-agent
     default_referer : string
@@ -155,21 +175,24 @@ def config(
     default_accept_language : string
         HTTP header accept-language
     nominatim_endpoint : string
-        which API endpoint to use for nominatim queries
+        the API endpoint to use for nominatim queries
     nominatim_key : string
         your API key, if you are using an endpoint that requires one
     overpass_endpoint : string
-        which API endpoint to use for overpass queries
-    all_oneway : boolean
-        if True, forces all paths to be loaded as oneway ways, preserving
+        the API endpoint to use for overpass queries
+    all_oneway : bool
+        if True, forces all ways to be loaded as oneway ways, preserving
         the original order of nodes stored in the OSM way XML. Only use if
-        specifically saving to .osm xml file with save_graph_xml function.
+        specifically saving to .osm XML file with save_graph_xml function.
+    elevation_provider : string
+        the API provider to use for adding node elevations, can be either
+        "google" or "airmap"
 
     Returns
     -------
     None
     """
-    # set each global variable to the passed-in parameter value
+    # set each global setting to the passed-in value
     settings.use_cache = use_cache
     settings.cache_folder = cache_folder
     settings.data_folder = data_folder
@@ -181,13 +204,15 @@ def config(
     settings.log_name = log_name
     settings.log_filename = log_filename
     settings.useful_tags_node = useful_tags_node
-    settings.useful_tags_path = useful_tags_path
-    settings.useful_tags_node = list(set(useful_tags_node + osm_xml_node_attrs + osm_xml_node_tags))
-    settings.useful_tags_path = list(set(useful_tags_path + osm_xml_way_attrs + osm_xml_way_tags))
+    settings.useful_tags_way = useful_tags_way
     settings.osm_xml_node_attrs = osm_xml_node_attrs
     settings.osm_xml_node_tags = osm_xml_node_tags
     settings.osm_xml_way_attrs = osm_xml_way_attrs
     settings.osm_xml_way_tags = osm_xml_way_tags
+    settings.overpass_settings = overpass_settings
+    settings.timeout = timeout
+    settings.memory = memory
+    settings.max_query_area_size = max_query_area_size
     settings.default_access = default_access
     settings.default_crs = default_crs
     settings.default_user_agent = default_user_agent
@@ -197,6 +222,7 @@ def config(
     settings.nominatim_key = nominatim_key
     settings.overpass_endpoint = overpass_endpoint
     settings.all_oneway = all_oneway
+    settings.elevation_provider = elevation_provider
 
     # if logging is turned on, log that we are configured
     if settings.log_file or settings.log_console:
@@ -207,8 +233,8 @@ def log(message, level=None, name=None, filename=None):
     """
     Write a message to the logger.
 
-    This logs to file and/or prints to the console, depending on the current
-    configuration of settings.log_file and settings.log_console.
+    This logs to file and/or prints to the console (terminal), depending on
+    the current configuration of settings.log_file and settings.log_console.
 
     Parameters
     ----------
