@@ -1,10 +1,8 @@
 """Geospatial utility functions."""
 
 import math
-from collections import OrderedDict
 
 import numpy as np
-import pandas as pd
 from shapely.geometry import LineString
 from shapely.geometry import MultiLineString
 from shapely.geometry import MultiPoint
@@ -13,46 +11,11 @@ from shapely.geometry import Point
 from shapely.geometry import Polygon
 from shapely.ops import split
 
-from . import downloader
 from . import projection
+from . import settings
 from . import utils
 
 
-def geocode(query):
-    """
-    Geocode a query string to (lat, lng) with the Nominatim geocoder.
-
-    Parameters
-    ----------
-    query : string
-        the query string to geocode
-
-    Returns
-    -------
-    point : tuple
-        the (lat, lng) coordinates returned by the geocoder
-    """
-    # define the parameters
-    params = OrderedDict()
-    params["format"] = "json"
-    params["limit"] = 1
-    params[
-        "dedupe"
-    ] = 0  # prevent OSM from deduping results so we get precisely 'limit' # of results
-    params["q"] = query
-    response_json = downloader.nominatim_request(params=params, timeout=30)
-
-    # if results were returned, parse lat and long out of the result
-    if len(response_json) > 0 and "lat" in response_json[0] and "lon" in response_json[0]:
-        lat = float(response_json[0]["lat"])
-        lng = float(response_json[0]["lon"])
-        point = (lat, lng)
-        utils.log(f'Geocoded "{query}" to {point}')
-        return point
-    else:
-        raise Exception(f'Nominatim geocoder returned no results for query "{query}"')
-
-
 def redistribute_vertices(geom, dist):
     """
     Redistribute the vertices on a projected LineString or MultiLineString.
@@ -214,9 +177,9 @@ def round_geometry_coords(shape, precision):
 
     Parameters
     ----------
-    shape : shapely.geometry.geometry, either Point, MultiPoint, LineString,
-            MultiLineString, Polygon, or MultiPolygon
-        the geometry to round the coordinates of
+    shape : shapely.geometry.geometry
+        the geometry to round the coordinates of; must be one of {Point,
+        MultiPoint, LineString, MultiLineString, Polygon, MultiPolygon}
     precision : int
         decimal precision to round coordinates to
 
@@ -246,25 +209,30 @@ def round_geometry_coords(shape, precision):
         raise TypeError(f"cannot round coordinates of unhandled geometry type: {type(shape)}")
 
 
-def _consolidate_subdivide_geometry(geometry, max_query_area_size):
+def _consolidate_subdivide_geometry(geometry, max_query_area_size=None):
     """
     Consolidate and subdivide some geometry.
 
     Consolidate a geometry into a convex hull, then subdivide it into smaller
-    sub-polygons if its area exceeds max size (in geometry's units).
+    sub-polygons if its area exceeds max size (in geometry's units). Configure
+    the max size via max_query_area_size in the settings module.
 
     Parameters
     ----------
     geometry : shapely.geometry.Polygon or shapely.geometry.MultiPolygon
         the geometry to consolidate and subdivide
-    max_query_area_size : float
-        max area for any part of the geometry in geometry's units:
-        any polygon bigger will get divided up for multiple queries to API
+    max_query_area_size : int
+        maximum area for any part of the geometry in meters: any polygon
+        bigger than this will get divided up for multiple queries to API
+        (default 50km x 50km). if None, use settings.max_query_area_size
 
     Returns
     -------
     geometry : shapely.geometry.Polygon or shapely.geometry.MultiPolygon
     """
+    if max_query_area_size is None:
+        max_query_area_size = settings.max_query_area_size
+
     # let the linear length of the quadrats (with which to subdivide the
     # geometry) be the square root of max area size
     quadrat_width = math.sqrt(max_query_area_size)
@@ -369,22 +337,22 @@ def _quadrat_cut_geometry(geometry, quadrat_width, min_num=3):
     return geometry
 
 
-def _intersect_index_quadrats(gdf, geometry, quadrat_width=0.05, min_num=3):
+def _intersect_index_quadrats(geometries, polygon, quadrat_width=0.05, min_num=3):
     """
-    Intersect points with a polygon.
+    Identify geometries that intersect a (multi)polygon.
 
     Use an r-tree spatial index and cut the polygon up into smaller
     sub-polygons for r-tree acceleration.
 
     Parameters
     ----------
-    gdf : geopandas.GeoDataFrame
-        the set of points to intersect
-    geometry : shapely.geometry.Polygon or shapely.geometry.MultiPolygon
-        the geometry to intersect with the points
+    geometries : geopandas.GeoSeries
+        the geometries to intersect with the polygon
+    polygon : shapely.geometry.Polygon or shapely.geometry.MultiPolygon
+        the polygon to intersect with the geometries
     quadrat_width : numeric
-        the linear length (in units the geometry is in) of the quadrats with
-        which to cut up the geometry (default = 0.05 degrees, approx 4km at
+        the linear length (in units the polygon is in) of the quadrats with
+        which to cut up the polygon (default = 0.05 degrees, approx 4km at
         NYC's latitude)
     min_num : int
         the minimum number of linear quadrat lines (e.g., min_num=3 would
@@ -392,93 +360,73 @@ def _intersect_index_quadrats(gdf, geometry, quadrat_width=0.05, min_num=3):
 
     Returns
     -------
-    points_within_geometry : geopandas.GeoDataFrame
+    geoms_in_poly : set
+        index labels of geometries that intersected polygon
     """
-    # create an empty dataframe to append matches to
-    points_within_geometry = pd.DataFrame()
+    # create an r-tree spatial index for the geometries
+    sindex = geometries.sindex
+    utils.log(f"Created r-tree spatial index for {len(geometries)} geometries")
 
-    # cut the geometry into chunks for r-tree spatial index intersecting
-    multipoly = _quadrat_cut_geometry(geometry, quadrat_width=quadrat_width, min_num=min_num)
+    # cut the polygon into chunks for spatial index intersecting
+    multipoly = _quadrat_cut_geometry(polygon, quadrat_width=quadrat_width, min_num=min_num)
+    geoms_in_poly = set()
 
-    # create an r-tree spatial index for the nodes (ie, points)
-    sindex = gdf["geometry"].sindex
-    utils.log(f"Created r-tree spatial index for {len(gdf)} points")
-
-    # loop through each chunk of the geometry to find approximate and then
-    # precisely intersecting points
+    # loop through each chunk of the polygon to find intersecting geometries
     for poly in multipoly:
+        # first find approximate matches with spatial index, then precise
+        # matches from those approximate ones
         poly = poly.buffer(0)
-
-        # find approximate matches with r-tree, then precise matches from those
-        # approximate ones
         if poly.is_valid and poly.area > 0:
-            possible_matches_index = list(sindex.intersection(poly.bounds))
-            possible_matches = gdf.iloc[possible_matches_index]
+            possible_matches_iloc = sindex.intersection(poly.bounds)
+            possible_matches = geometries.iloc[list(possible_matches_iloc)]
             precise_matches = possible_matches[possible_matches.intersects(poly)]
-            points_within_geometry = points_within_geometry.append(precise_matches)
+            geoms_in_poly.update(precise_matches.index)
 
-    if len(points_within_geometry) > 0:
-        # drop duplicate points if any
-        points_within_geometry = points_within_geometry.drop_duplicates(subset="node")
-    else:
-        # after simplifying the graph, and given the requested network type,
-        # there are no nodes inside the polygon - can't create graph from that
-        # so throw error
-        raise Exception("There are no nodes within the requested geometry")
-
-    utils.log(f"Identified {len(points_within_geometry)} nodes inside polygon")
-    return points_within_geometry
+    utils.log(f"Identified {len(geoms_in_poly)} geometries inside polygon")
+    return geoms_in_poly
 
 
 def bbox_from_point(point, dist=1000, project_utm=False, return_crs=False):
     """
-    Create a bounding box from a point.
+    Create a bounding box from a (lat, lng) center point.
 
     Create a bounding box some distance in each direction (north, south, east,
-    and west) from some (lat, lng) point.
+    and west) from the center point and optionally project it.
 
     Parameters
     ----------
     point : tuple
-        the (lat, lng) point to create the bounding box around
+        the (lat, lng) center point to create the bounding box around
     dist : int
-        how many meters the north, south, east, and west sides of the box should
-        each be from the point
+        bounding box distance in meters from the center point
     project_utm : bool
-        if True return bbox as UTM coordinates
+        if True, return bounding box as UTM-projected coordinates
     return_crs : bool
-        if True and project_utm=True, return the projected CRS
+        if True, and project_utm=True, return the projected CRS too
 
     Returns
     -------
     tuple
-        (north, south, east, west) if return_crs=False or
-        (north, south, east, west, crs_proj) if return_crs=True
+        (north, south, east, west) or (north, south, east, west, crs_proj)
     """
-    # reverse the order of the (lat,lng) point so it is (x,y) for shapely, then
-    # project to UTM and buffer in meters
+    earth_radius = 6371009  # meters
     lat, lng = point
-    point_proj, crs_proj = projection.project_geometry(Point((lng, lat)))
-    buffer_proj = point_proj.buffer(dist)
+
+    delta_lat = (dist / earth_radius) * (180 / math.pi)
+    delta_lng = (dist / earth_radius) * (180 / math.pi) / math.cos(lat * math.pi / 180)
+    north = lat + delta_lat
+    south = lat - delta_lat
+    east = lng + delta_lng
+    west = lng - delta_lng
 
     if project_utm:
-        west, south, east, north = buffer_proj.bounds
-        utils.log(
-            f"Created bbox {dist} m from {point} and projected it: {north},{south},{east},{west}"
-        )
-    else:
-        # if project_utm is False, project back to lat-lng then get the
-        # bounding coordinates
-        buffer_latlong, _ = projection.project_geometry(buffer_proj, crs=crs_proj, to_latlong=True)
-        west, south, east, north = buffer_latlong.bounds
-        utils.log(
-            (
-                f"Created bounding box {dist} meters in each direction "
-                f"from {point}: {north},{south},{east},{west}"
-            )
-        )
+        bbox_poly = bbox_to_poly(north, south, east, west)
+        bbox_proj, crs_proj = projection.project_geometry(bbox_poly)
+        west, south, east, north = bbox_proj.bounds
 
-    if return_crs:
+    utils.log(f"Created bbox {dist} m from {point}: {north},{south},{east},{west}")
+
+    if project_utm and return_crs:
         return north, south, east, west, crs_proj
     else:
         return north, south, east, west
