@@ -1,7 +1,7 @@
 """Graph utility functions."""
 
+import itertools
 from collections import Counter
-from itertools import chain
 
 import geopandas as gpd
 import networkx as nx
@@ -16,7 +16,9 @@ from . import utils
 
 def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geometry=True):
     """
-    Convert a graph into node and/or edge GeoDataFrames.
+    Convert a graph to node and/or edge GeoDataFrames.
+
+    This function is the inverse of `graph_from_gdfs`.
 
     Parameters
     ----------
@@ -29,110 +31,117 @@ def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geome
     node_geometry : bool
         if True, create a geometry column from node x and y data
     fill_edge_geometry : bool
-        if True, fill in missing edge geometry fields using origin and
-        destination nodes
+        if True, fill in missing edge geometry fields using nodes u and v
 
     Returns
     -------
     geopandas.GeoDataFrame or tuple
         gdf_nodes or gdf_edges or tuple of (gdf_nodes, gdf_edges)
     """
-    if not (nodes or edges):
-        raise ValueError("You must request nodes or edges, or both.")
-
-    to_return = []
+    crs = G.graph["crs"]
 
     if nodes:
 
         nodes, data = zip(*G.nodes(data=True))
-        gdf_nodes = gpd.GeoDataFrame(list(data), index=nodes)
-        if node_geometry:
-            gdf_nodes["geometry"] = gdf_nodes.apply(lambda row: Point(row["x"], row["y"]), axis=1)
-            gdf_nodes.set_geometry("geometry", inplace=True)
-        gdf_nodes.crs = G.graph["crs"]
 
-        to_return.append(gdf_nodes)
+        if node_geometry:
+            # convert node x/y attributes to Points for geometry column
+            geom = (Point(d["x"], d["y"]) for d in data)
+            gdf_nodes = gpd.GeoDataFrame(data, index=nodes, crs=crs, geometry=list(geom))
+        else:
+            gdf_nodes = gpd.GeoDataFrame(data, index=nodes)
+
         utils.log("Created nodes GeoDataFrame from graph")
 
     if edges:
 
-        # create a list to hold our edges, then loop through each edge in the
-        # graph
-        edges = []
-        for u, v, key, data in G.edges(keys=True, data=True):
+        if len(G.edges) < 1:
+            raise ValueError("Graph has no edges, cannot convert to a GeoDataFrame.")
 
-            # for each edge, add key and all attributes in data dict to the
-            # edge_details
-            edge_details = {"u": u, "v": v, "key": key}
-            for attr_key in data:
-                edge_details[attr_key] = data[attr_key]
+        u, v, k, data = zip(*G.edges(keys=True, data=True))
 
-            # if edge doesn't already have a geometry attribute, create one now
-            # if fill_edge_geometry==True
-            if "geometry" not in data:
-                if fill_edge_geometry:
-                    point_u = Point((G.nodes[u]["x"], G.nodes[u]["y"]))
-                    point_v = Point((G.nodes[v]["x"], G.nodes[v]["y"]))
-                    edge_details["geometry"] = LineString([point_u, point_v])
+        if fill_edge_geometry:
+
+            # subroutine to get geometry for every edge: if edge already has
+            # geometry return it, otherwise create it using the incident nodes
+            x_lookup = nx.get_node_attributes(G, "x")
+            y_lookup = nx.get_node_attributes(G, "y")
+
+            def make_geom(u, v, data, x=x_lookup, y=y_lookup):
+                if "geometry" in data:
+                    return data["geometry"]
                 else:
-                    edge_details["geometry"] = np.nan
+                    return LineString((Point((x[u], y[u])), Point((x[v], y[v]))))
 
-            edges.append(edge_details)
+            geom = map(make_geom, u, v, data)
+            gdf_edges = gpd.GeoDataFrame(data, crs=crs, geometry=list(geom))
 
-        # create a GeoDataFrame from the list of edges and set the CRS
-        gdf_edges = gpd.GeoDataFrame(edges)
-        gdf_edges.crs = G.graph["crs"]
+        else:
+            gdf_edges = gpd.GeoDataFrame(data)
+            if "geometry" not in gdf_edges.columns:
+                # if no edges have a geometry attribute, create null column
+                gdf_edges["geometry"] = np.nan
+            gdf_edges.set_geometry("geometry")
+            gdf_edges.crs = crs
+
+        # add u, v, key attributes as columns
+        gdf_edges["u"] = u
+        gdf_edges["v"] = v
+        gdf_edges["key"] = k
 
-        to_return.append(gdf_edges)
         utils.log("Created edges GeoDataFrame from graph")
 
-    if len(to_return) > 1:
-        return tuple(to_return)
+    if nodes and edges:
+        return gdf_nodes, gdf_edges
+    elif nodes:
+        return gdf_nodes
+    elif edges:
+        return gdf_edges
     else:
-        return to_return[0]
+        raise ValueError("You must request nodes or edges or both.")
 
 
-def graph_from_gdfs(gdf_nodes, gdf_edges):
+def graph_from_gdfs(gdf_nodes, gdf_edges, graph_attrs=None):
     """
-    Convert node and edge GeoDataFrames into a MultiDiGraph.
+    Convert node and edge GeoDataFrames to a MultiDiGraph.
+
+    This function is the inverse of `graph_to_gdfs`.
 
     Parameters
     ----------
     gdf_nodes : geopandas.GeoDataFrame
         GeoDataFrame of graph nodes
     gdf_edges : geopandas.GeoDataFrame
-        GeoDataFrame of graph edges
+        GeoDataFrame of graph edges, must have crs attribute set
+    graph_attrs : dict
+        the new G.graph attribute dict; if None, add crs as the only
+        graph-level attribute
 
     Returns
     -------
     G : networkx.MultiDiGraph
     """
-    G = nx.MultiDiGraph()
-    G.graph["crs"] = gdf_nodes.crs
+    if graph_attrs is None:
+        graph_attrs = {"crs": gdf_edges.crs}
+    G = nx.MultiDiGraph(**graph_attrs)
 
-    # add the nodes and their attributes to the graph
+    # add the nodes then each node's non-null attributes
     G.add_nodes_from(gdf_nodes.index)
-    attributes = gdf_nodes.to_dict()
-    for attribute_name in gdf_nodes.columns:
-        # only add this attribute to nodes which have a non-null value for it
-        attribute_values = {k: v for k, v in attributes[attribute_name].items() if pd.notnull(v)}
-        nx.set_node_attributes(G, name=attribute_name, values=attribute_values)
+    for col in gdf_nodes.columns:
+        nx.set_node_attributes(G, name=col, values=gdf_nodes[col].dropna())
 
-    # add the edges and attributes that are not u, v, key (as they're added
-    # separately) or null
-    for _, row in gdf_edges.iterrows():
-        attrs = {}
-        for label, value in row.items():
-            if (label not in ["u", "v", "key"]) and (isinstance(value, list) or pd.notnull(value)):
-                attrs[label] = value
-        G.add_edge(row["u"], row["v"], key=row["key"], **attrs)
+    # add each edge and its non-null attributes
+    for (u, v, k), row in gdf_edges.set_index(["u", "v", "key"]).iterrows():
+        d = {label: val for label, val in row.items() if isinstance(val, list) or pd.notnull(val)}
+        G.add_edge(u, v, k, **d)
 
+    utils.log("Created graph from node/edge GeoDataFrames")
     return G
 
 
 def induce_subgraph(G, node_subset):
     """
-    Induce a subgraph of G.
+    Induce a subgraph of G: deprecated, do not use.
 
     Parameters
     ----------
@@ -143,37 +152,19 @@ def induce_subgraph(G, node_subset):
 
     Returns
     -------
-    H : networkx.MultiDiGraph
+    networkx.MultiDiGraph
         the subgraph of G induced by node_subset
     """
-    node_subset = set(node_subset)
+    import warnings
 
-    # copy nodes into new graph
-    H = G.__class__()
-    H.add_nodes_from((n, G.nodes[n]) for n in node_subset)
+    msg = (
+        "The induce_subgraph function has been deprecated and will be "
+        "removed in a future release. Use G.subgraph(nodes) instead."
+    )
+    warnings.warn(msg)
 
-    # copy edges to new graph, including parallel edges
-    if H.is_multigraph:
-        H.add_edges_from(
-            (n, nbr, key, d)
-            for n, nbrs in G.adj.items()
-            if n in node_subset
-            for nbr, keydict in nbrs.items()
-            if nbr in node_subset
-            for key, d in keydict.items()
-        )
-    else:
-        H.add_edges_from(
-            (n, nbr, d)
-            for n, nbrs in G.adj.items()
-            if n in node_subset
-            for nbr, d in nbrs.items()
-            if nbr in node_subset
-        )
-
-    # update graph attribute dict, and return graph
-    H.graph.update(G.graph)
-    return H
+    # induce a (frozen) subgraph then unfreeze it by making new MultiDiGraph
+    return nx.MultiDiGraph(G.subgraph(node_subset))
 
 
 def get_largest_component(G, strongly=False):
@@ -191,38 +182,25 @@ def get_largest_component(G, strongly=False):
     Returns
     -------
     G : networkx.MultiDiGraph
-        the largest connected component subgraph from the original graph
+        the largest connected component subgraph of the original graph
     """
-    original_len = len(list(G.nodes()))
-
     if strongly:
-        # if the graph is not connected retain only the largest strongly connected component
-        if not nx.is_strongly_connected(G):
-
-            # get all the strongly connected components in graph then identify the largest
-            sccs = nx.strongly_connected_components(G)
-            largest_scc = max(sccs, key=len)
-            G = induce_subgraph(G, largest_scc)
-
-            msg = (
-                f"Graph was not connected, retained only the largest strongly "
-                f"connected component ({len(G)} of {original_len} total nodes)"
-            )
-            utils.log(msg)
+        kind = "strongly"
+        is_connected = nx.is_strongly_connected
+        connected_components = nx.strongly_connected_components
     else:
-        # if the graph is not connected retain only the largest weakly connected component
-        if not nx.is_weakly_connected(G):
+        kind = "weakly"
+        is_connected = nx.is_weakly_connected
+        connected_components = nx.weakly_connected_components
 
-            # get all the weakly connected components in graph then identify the largest
-            wccs = nx.weakly_connected_components(G)
-            largest_wcc = max(wccs, key=len)
-            G = induce_subgraph(G, largest_wcc)
+    if not is_connected(G):
+        # get all the connected components in graph then identify the largest
+        largest_cc = max(connected_components(G), key=len)
+        n = len(G)
 
-            msg = (
-                f"Graph was not connected, retained only the largest weakly "
-                f"connected component ({len(G)} of {original_len} total nodes)"
-            )
-            utils.log(msg)
+        # induce (frozen) subgraph then unfreeze it by making new MultiDiGraph
+        G = nx.MultiDiGraph(G.subgraph(largest_cc))
+        utils.log(f"Got largest {kind} connected component ({len(G)} of {n} total nodes)")
 
     return G
 
@@ -238,17 +216,18 @@ def get_route_edge_attributes(
     G : networkx.MultiDiGraph
         input graph
     route : list
-        list of nodes in the path
+        list of nodes IDs constituting the path
     attribute : string
-        the name of the attribute to get the value of for each edge.
-        If not specified, the complete data dict is returned for each edge.
+        the name of the attribute to get the value of for each edge. If None,
+        the complete data dict is returned for each edge.
     minimize_key : string
         if there are parallel edges between two nodes, select the one with the
         lowest value of minimize_key
     retrieve_default : Callable[Tuple[Any, Any], Any]
-        Function called with the edge nodes as parameters to retrieve a
-        default value, if the edge does not contain the given attribute. Per
-        default, a `KeyError` is raised
+        function called with the edge nodes as parameters to retrieve a
+        default value, if the edge does not contain the given attribute
+        (otherwise a `KeyError` is raised)
+
     Returns
     -------
     attribute_values : list
@@ -323,16 +302,12 @@ def count_streets_per_node(G, nodes=None):
     edges = non_self_loop_edges + self_loop_edges
 
     # flatten the list of (u,v) tuples
-    edges_flat = list(chain.from_iterable(edges))
+    edges_flat = list(itertools.chain.from_iterable(edges))
 
     # count how often each node appears in the list of flattened edge endpoints
     counts = Counter(edges_flat)
     streets_per_node = {node: counts[node] for node in nodes}
-    msg = (
-        "Got the counts of undirected street segments incident to each node "
-        "(before removing peripheral edges)"
-    )
-    utils.log(msg)
+    utils.log("Counted undirected street segments incident to each node")
     return streets_per_node
 
 
@@ -343,14 +318,14 @@ def remove_isolated_nodes(G):
     Parameters
     ----------
     G : networkx.MultiDiGraph
-        graph from which to remove nodes
+        graph from which to remove isolated nodes
 
     Returns
     -------
     G : networkx.MultiDiGraph
         graph with all isolated nodes removed
     """
-    isolated_nodes = [node for node, degree in dict(G.degree()).items() if degree < 1]
+    isolated_nodes = [node for node, degree in G.degree() if degree < 1]
     G.remove_nodes_from(isolated_nodes)
     utils.log(f"Removed {len(isolated_nodes)} isolated nodes")
     return G
@@ -489,11 +464,46 @@ def _update_edge_keys(G):
     return G
 
 
+def get_digraph(G, weight="length"):
+    """
+    Convert MultiDiGraph to DiGraph.
+
+    Chooses between parallel edges by minimizing `weight` attribute value.
+    Note: see also `get_undirected` to convert MultiDiGraph to MultiGraph.
+
+    Parameters
+    ----------
+    G : networkx.MultiDiGraph
+        input graph
+    weight : string
+        attribute value to minimize when choosing between parallel edges
+
+    Returns
+    -------
+    networkx.DiGraph
+    """
+    # make a copy to not edit the original graph object the caller passed in
+    G = G.copy()
+    to_remove = []
+
+    # identify all the parallel edges in the MultiDiGraph
+    parallels = ((u, v) for u, v, k in G.edges(keys=True) if k > 0)
+
+    # remove the parallel edge with greater "weight" attribute value
+    for u, v in set(parallels):
+        k, d = max(G.get_edge_data(u, v).items(), key=lambda x: x[1][weight])
+        to_remove.append((u, v, k))
+
+    G.remove_edges_from(to_remove)
+    return nx.DiGraph(G)
+
+
 def get_undirected(G):
     """
     Convert MultiDiGraph to MultiGraph.
 
-    Maintains parallel edges if their geometries differ.
+    Maintains parallel edges only if their geometries differ. Note: see also
+    `get_digraph` to convert MultiDiGraph to DiGraph.
 
     Parameters
     ----------
@@ -502,10 +512,12 @@ def get_undirected(G):
 
     Returns
     -------
-    H : networkx.MultiGraph
+    networkx.MultiGraph
     """
-    # set from/to nodes before making graph undirected
+    # make a copy to not edit the original graph object the caller passed in
     G = G.copy()
+
+    # set from/to nodes before making graph undirected
     for u, v, k, data in G.edges(keys=True, data=True):
         G.edges[u, v, k]["from"] = u
         G.edges[u, v, k]["to"] = v
@@ -527,7 +539,6 @@ def get_undirected(G):
     H.add_nodes_from(G.nodes(data=True))
     H.add_edges_from(G.edges(keys=True, data=True))
     H.graph = G.graph
-    H.name = G.name
 
     # the previous operation added all directed edges from G as undirected
     # edges in H. this means we have duplicate edges for every bi-directional
@@ -553,25 +564,29 @@ def get_undirected(G):
                         duplicate_edges.append((u, v, key_other))
 
     H.remove_edges_from(duplicate_edges)
-    utils.log("Made undirected graph")
+    utils.log("Converted MultiDiGraph to undirected MultiGraph")
 
     return H
 
 
-def add_edge_lengths(G):
+def add_edge_lengths(G, precision=3):
     """
-    Add length (meters) attribute to each edge.
+    Add `length` (meters) attribute to each edge.
 
-    Calculate via great circle distance between nodes u and v.
+    Calculated via great-circle distance between each edge's incident nodes,
+    so ensure graph is in unprojected coordinates.
 
     Parameters
     ----------
     G : networkx.MultiDiGraph
         input graph
+    precision : int
+        decimal precision to round lengths
 
     Returns
     -------
     G : networkx.MultiDiGraph
+        graph with edge length attributes
     """
     # first load all the edges' origin and destination coordinates as a
     # dataframe indexed by u, v, key
@@ -602,8 +617,8 @@ def add_edge_lengths(G):
         lat1=df_coords["u_y"], lng1=df_coords["u_x"], lat2=df_coords["v_y"], lng2=df_coords["v_x"]
     )
 
-    # fill nulls with zeros and round to the millimeter
-    gc_distances = gc_distances.fillna(value=0).round(3)
+    # fill nulls with zeros and round
+    gc_distances = gc_distances.fillna(value=0).round(precision)
     nx.set_edge_attributes(G, name="length", values=gc_distances.to_dict())
 
     utils.log("Added edge lengths to graph")
